Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid
dSS.devprops.AKMDelayConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;

        this.addEvents('formsubmitted', 'formloaded');

        this.onDelay = new Ext.Panel(
        {
            border: false,
            height: 35,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Turn-on delay") + ":",
                    width: 173,
                    style: { marginTop: '5px' }
                },
                {
                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'minutes',
                    minValue: 0,
                    maxValue: 100,
                    allowDecimals: false,
                    defaultValue: 0,
                    allowEmpty: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'seconds',
                    minValue: 0,
                    maxValue: 59,
                    allowDecimals: false,
                    defaultValue: 0,
                    allowEmpty: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: dSS._("mm:ss"),
                    style: { marginTop: '5px' }
                }

            ]
        });

        this.offDelay = new Ext.Panel(
        {
            border: false,
            height: 25,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Turn-off delay") + ":",
                    width: 173,
                    style: { marginTop: '5px' }
                },
                {
                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'minutes',
                    minValue: 0,
                    maxValue: 100,
                    allowDecimals: false,
                    defaultValue: 0,
                    allowEmpty: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'seconds',
                    minValue: 0,
                    maxValue: 59,
                    allowDecimals: false,
                    defaultValue: 0,
                    allowEmpty: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: dSS._("mm:ss"),
                    style: { marginTop: '5px' }
                }

            ]
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            scope: this,
            items:
            [
                this.onDelay,
                this.offDelay
            ]
        });

        dSS.devprops.AKMDelayConfigForm.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.AKM_DELAY_CONFIG) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.offDelay.minutes.markInvalid(dSS._(data.json.message));
                    this.offDelay.seconds.markInvalid(dSS._(data.json.message));
                    this.onDelay.minutes.markInvalid(dSS._(data.json.message));
                    this.onDelay.seconds.markInvalid(dSS._(data.json.message));
                    this.fireEvent('formloaded', true);
                    return;
                }

                this.loadedOnDelay = data.json.result.ondelay;
                this.loadedOffDelay = data.json.result.offdelay;

                var seconds = this.loadedOnDelay / 1000;
                var minutes = seconds / 60;
                seconds = seconds % 60;

                this.onDelay.minutes.setValue(Math.floor(minutes));
                this.onDelay.seconds.setValue(Math.floor(seconds));

                seconds = this.loadedOffDelay / 1000;
                minutes = seconds / 60;
                seconds = seconds % 60;

                this.offDelay.minutes.setValue(Math.floor(minutes));
                this.offDelay.seconds.setValue(Math.floor(seconds));

                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function(unmaskEl) {
            if (this.getForm().isValid() !== true) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Please correct values in the red " +
                                    "marked fields and try again"),
                              unmaskEl.unmask());
                return;
            }

            var minutes = parseInt(this.onDelay.minutes.getValue(), 10);
            var seconds = parseInt(this.onDelay.seconds.getValue(), 10);
            var onMs = (seconds + (minutes * 60)) * 1000;

            minutes = parseInt(this.offDelay.minutes.getValue(), 10);
            seconds = parseInt(this.offDelay.seconds.getValue(), 10);
            var offMs = (seconds + (minutes * 60)) * 1000;

            if ((this.loadedOnDelay === onMs) &&
                (this.loadedOffDelay === offMs)) {
                this.fireEvent('formsubmitted');
                return;
            }

            dSS.util.genericRequest.defer(2000, this,
            [
                {
                    url: '/json/device/setAKMInputTimeouts',
                    params:
                    {
                        dsid: this.dsid,
                        ondelay: onMs,
                        offdelay: offMs
                    },
                    failureMsg: dSS._("Could not save delay settings"),
                    scope: this,
                    evt: 'formsubmitted'
                }
            ]);
        },
        this);
    }
});

Ext.reg('dssakmdelayconfigform', dSS.devprops.AKMDelayConfigForm);

