Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid
dSS.devprops.AKMInputConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;

        this.addEvents('formsubmitted', 'formloaded');

        this.inputSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 173,
            mode: 'local',
            fieldLabel: dSS._("Input"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data:
                [
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_STANDARD,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_STANDARD])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_INVERTED,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_INVERTED])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_ON_RISINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_ON_RISINGEDGE])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_ON_FALLINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_ON_FALLINGEDGE])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_OFF_RISINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_OFF_RISINGEDGE])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_OFF_FALLINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_OFF_FALLINGEDGE])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_RISINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_RISINGEDGE])
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_AKM_FALLINGEDGE,
                        dSS._(dSS.config.AKMButtonInputStrings[dSS.config.ButtonInputMode.BUTTONINPUT_AKM_FALLINGEDGE])
                    ]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.inputSelection
                    ]
                }
            ]
        });

        dSS.devprops.AKMInputConfigForm.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
            if (this.closed === true) {
                return;
            }

            if (data.loader !== dSS.devprops.Loaders.AKM_INPUT_PROPERTY) {
                return;
            }

            if (data.json.ok !== true) {
                this.inputSelection.markInvalid(dSS._(data.json.message));
                this.fireEvent('formloaded', true);
                return;
            }

            this.loadedValue = data.json.result.value;
            this.inputSelection.setValue(this.loadedValue);
            this.fireEvent('formloaded', true);
        },
        this);

        this.on('submit', function() {
            var value = this.inputSelection.getValue();
            var dsid = this.dsid;
            if (value === this.loadedValue) {
                this.fireEvent('formsubmitted');
                return;
            }

            dSS.util.genericRequest.defer(1000, this,
            [
                {
                    url: '/json/device/setAKMInputProperty',
                    params:
                    {
                        dsid: this.dsid,
                        mode: value
                    },
                    failureMsg: dSS._("Could not save AKM input property"),
                    scope: this,
                    evt: 'formsubmitted'
                }
            ]);
        },
        this);
    }
});

Ext.reg('dssakminputconfigform', dSS.devprops.AKMInputConfigForm);

