Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid
dSS.devprops.AKMSensorConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;

        this.addEvents('formsubmitted', 'formloaded');

        this.sensorSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 173,
            mode: 'local',
            fieldLabel: dSS._("Sensor function"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data:
                [
                    [
                        0,
                        dSS._(dSS.config.AKMInputTypeStrings[0])
                    ],
                    [
                        4,
                        dSS._(dSS.config.AKMInputTypeStrings[4])
                    ],
                    [
                        5,
                        dSS._(dSS.config.AKMInputTypeStrings[5])
                    ],
                    [
                        7,
                        dSS._(dSS.config.AKMInputTypeStrings[7])
                    ],
                    [
                        8,
                        dSS._(dSS.config.AKMInputTypeStrings[8])
                    ],
                    [
                        9,
                        dSS._(dSS.config.AKMInputTypeStrings[9])
                    ],
                    [
                        10,
                        dSS._(dSS.config.AKMInputTypeStrings[10])
                    ],
                    [
                        11,
                        dSS._(dSS.config.AKMInputTypeStrings[11])
                    ]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.sensorSelection
                    ]
                }
            ]
        });

        dSS.devprops.AKMSensorConfigForm.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.AKM_BINARY_INPUTS) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.sensorSelection.markInvalid(dSS._(data.json.message));
                    this.fireEvent('formloaded', true);
                    return;
                }

                var inputs = data.json.result.inputs;
                if (inputs.length < 1) {
                    this.sensorSelection.markInvalid(dSS._(data.json.message));
                    this.fireEvent('formloaded', true);
                }

                this.loadedValue = inputs[0].inputType;
                this.sensorSelection.setValue(this.loadedValue);
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function() {
            var value = this.sensorSelection.getValue();
            var dsid = this.dsid;
            if (value === this.loadedValue) {
                this.fireEvent('formsubmitted');
                return;
            }

            dSS.util.genericRequest({
                url: '/json/device/setBinaryInputType',
                params:
                {
                    dsid: this.dsid,
                    index: 0,
                    type: value
                },
                failureMsg: dSS._("Could not save sensor function"),
                scope: this,
                evt: 'formsubmitted'
            });
        },
        this);
    }
});

Ext.reg('dssakmsensorconfigform', dSS.devprops.AKMSensorConfigForm);

