Ext.namespace("dSS", "dSS.devices", "dSS.config", "dSS.gthelper");

// group
//  stop button will be shown depending on the group
dSS.devices.DeviceSettingsPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.addEvents('loading');

        this.dsid = config.dsid;
        this.previousBladeAngle = -1;

        this.maxValue = 0xff;
        var decPrecision = 0;

        if (config.matrix.shadeposition === true) {
            this.maxValue = 0xffff;
            decPrecision = 2;
        }

        this.matrix = config.matrix;

        this.DEFAULT_HEIGHT = 176;
        this.EXTRA_HEIGHT = 206;
        var initialHeight = this.DEFAULT_HEIGHT;

        if (this.matrix.shadebladeang === true) {
            initialHeight = this.EXTRA_HEIGHT;
        }

        var xPos = dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth');

        this.valueLabel = new Ext.form.Label({
            x: 10,
            y: 16,
            text: dSS._("Value") + ':'
        });

        this.valueField = new Ext.form.NumberField({
            x: xPos - 200,
            y: 12,
            width: 50,
            allowBlank: false,
            allowNegative: false,
            allowDecimals: (decPrecision !== 0),
            decimalPrecision: decPrecision,
            forceDecimalPrecision: true,
            value: 0,
            minValue: 0,
            maxValue: 100,
            listeners: {
                change: this.updateSlider.createDelegate(this)
            }
        });

        this.bladeAngleLabel = new Ext.form.Label({
            x: 10,
            y: 178,
            text: dSS._("Set blade angle") + ':'
        });

        this.bladeAngleField = new Ext.form.NumberField({
            x: xPos - 200,
            y: 174,
            width: 50,
            allowBlank: false,
            allowNegative: false,
            allowDecimals: false,
            value: 0,
            minValue: 0,
            maxValue: 100,
            listeners: {
                change: this.updateAngleSlider.createDelegate(this)
            }
        });

        this.bladeAngleUnit = new Ext.form.Label({
            x: xPos - 145,
            y: 178,
            text: '%'
        });

        this.angleSlider = new Ext.Slider({
            width: 78,
            minValue: 0,
            increment: 1,
            maxValue: 100,
            vertical: false,
            decimalPrecision: 1,
            plugins: new Ext.slider.Tip({
                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleAngleSliderChanged.createDelegate(this)
            }
        });

        this.valueSlider = new Ext.Slider({
            height: 150,
            minValue: 0,
            increment: (decPrecision === 0) ? 1 : 0.001,
            maxValue: 100,
            vertical: true,
            decimalPrecision: decPrecision,
            plugins: new Ext.slider.Tip({
                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleSliderChangedProxy.createDelegate(this)
            }
        });

        this.stopButton = new Ext.Button({
            x: xPos - 200,
            y: 140,
            xtype: 'button',
            text: dSS._("Stop"),
            width: 50,
            handler: this.handleStop.createDelegate(this)
        });

        this.maxButton = new Ext.Button({
            xtype: 'button',
            x: xPos - 98,
            y: 12,
            text: dSS._("Max"),
            tooltip: dSS._("Set maximum value"),
            width: 50,
            handler: this.handleMax.createDelegate(this)
        });

        this.minButton = new Ext.Button({
            x: xPos - 98,
            y: 140,
            xtype: 'button',
            text: dSS._("Min"),
            tooltip: dSS._("Set minimum value"),
            width: 50,
            handler: this.handleMin.createDelegate(this)
        });

        this.dirtyValue = false;
        this.dirtyAngle = false;

        Ext.apply(this, {
            layout: 'absolute',
            padding: 5,
            bubbleEvents: [ 'updateshadow', 'valuechanged' ],
            height: initialHeight,
            items:
            [
                this.valueLabel,
                this.valueField,
                {
                    x: xPos - 145,
                    y: 16,
                    xtype: 'label',
                    text: '%'
                },
                {
                    x: xPos - 130,
                    y: 12,
                    xtype: 'panel',
                    border: false,
                    layout: 'form',
                    items: [ this.valueSlider ]
                },
                this.maxButton,
                {
                    x: xPos - 98,
                    y: 54,
                    xtype: 'button',
                    iconCls: 'buttonUpIcon',
                    tooltip: dSS._("Increase value"),
                    width: 50,
                    handler: this.handleInc.createDelegate(this)
                },
                {
                    x: xPos - 98,
                    y: 98,
                    xtype: 'button',
                    iconCls: 'buttonDownIcon',
                    tooltip: dSS._("Decrease value"),
                    width: 50,
                    handler: this.handleDec.createDelegate(this)
                },
                this.stopButton,
                this.minButton,
                this.bladeAngleLabel,
                this.bladeAngleField,
                this.bladeAngleUnit,
                {
                    x: xPos - 126,
                    y: 174,
                    xtype: 'panel',
                    border: false,
                    layout: 'form',
                    items: [ this.angleSlider ]
                }
            ]
        });

        this.setData(this.dsid, config.group, config.present, config.matrix);

        dSS.devices.DeviceSettingsPanel.superclass.constructor.apply(this, arguments);

        this.addEvents('updateshadow');

        this.on('afterrender', function() {
                this.fireEvent('updateshadow', initialHeight);
            },
            this);

        this.on('beforedestroy', function() {
                this.killRequest();
                this.closed = true;
            },
            this);

        this.valueField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = parseFloat(field.getValue());
                    var ov = parseFloat(this.valueSlider.getValue());
                    if (nv !== ov) {
                        this.updateSlider(field, nv);
                    }
                }
            },
            this);

        this.bladeAngleField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = field.getValue();
                    this.updateAngleSlider(field, nv,
                                           this.angleSlider.getValue());
                }
            },
            this);
    },
    percentToValue: function(percent) {
        var value = percent / 100;
        value = value * this.maxValue;
        if (value < 0) {
            value = 0;
        } else if (value > this.maxValue) {
            value = this.maxValue;
        }
        return Math.round(value);
    },
    valueToPercent: function(value) {
        if (value < 0) {
            value = 0;
        }  else if (value > this.maxValue) {
            value = this.maxValue;
        }
        return (value / this.maxValue * 100);
    },
    handleSliderChangedProxy: function(slider, newVal, thumb) {
        this.fireEvent('valuechanged', this.dsid);
        this.handleSliderChanged(slider, newVal, thumb);
    },
    handleSliderChanged: function(slider, newVal, thumb) {
        this.markAndAbortRead();
        this.valueField.setValue(newVal);
        if (this.maxValue <= 0xff) {
            dSS.util.setValueOnDevice(this.dsid, this.percentToValue(newVal));
        } else {
            dSS.util.setOutputValueOnDevice(this.dsid, 0x02,
                                            this.percentToValue(newVal));
        }
        if (this.matrix.shadebladeang === true) {
            if (newVal >= 90) {
                var oldAngle = this.bladeAngleField.getValue();
                this.bladeAngleField.setValue(100);
                this.updateAngleSlider.defer(300, this,
                        [ this.bladeAngleField, 100, oldAngle ]);
            }
        }
    },
    handleAngleSliderChanged: function(slider, newVal, thumb) {
        this.fireEvent('valuechanged', this.dsid);
        this.markAndAbortRead();
        this.bladeAngleField.setValue(newVal);
        this.handleBladeAngleChanged(this.bladeAngleField, newVal,
                                     this.previousBladeAngle);
    },
    handleBladeAngleChanged: function(field, newVal, oldVal) {
        this.markAndAbortReadAngle();
        if (newVal !== this.previousBladeAngle) {
            dSS.util.genericRequest({
                    url: '/json/device/setConfig',
                    scope: this,
                    params:
                    {
                        'class': 64,
                        index: 0x04,
                        dsid: this.dsid,
                        value: Math.round(newVal / 100 * 0xff)
                    },
                    failureMsg: dSS._("Could not set device blade angle")
                });
            this.previousBladeAngle = newVal;
            this.dirtyAngle = false;
        }
    },
    updateAngleSlider: function(field, newVal, oldVal) {
        this.fireEvent('valuechanged', this.dsid);
        this.markAndAbortRead();
        var ov = this.angleSlider.getValue();
        if ((newVal !== oldVal) && (newVal !== ov)) {
            this.angleSlider.setValue(newVal);
            this.handleBladeAngleChanged(field, newVal, oldVal);
        }
    },
    updateSlider: function(field, newVal) {
        this.fireEvent('valuechanged', this.dsid);
        this.markAndAbortRead();
        var oldVal = this.valueSlider.getValue();
        if (parseFloat(newVal) !== parseFloat(oldVal)) {
            this.valueSlider.setValue(newVal);
            if (this.maxValue <= 0xff) {
                dSS.util.setValueOnDevice(this.dsid,
                                          this.percentToValue(newVal));
            } else {
                dSS.util.setOutputValueOnDevice(this.dsid, 0x02,
                                                this.percentToValue(newVal));
            }
            if (this.matrix.shadebladeang === true) {
                if (newVal >= 90) {
                    var oldAngle = this.bladeAngleField.getValue();
                    this.bladeAngleField.setValue(100);
                    this.updateAngleSlider.defer(300, this,
                            [ this.bladeAngleField, 100, oldAngle ]);
                }
            }
        }
    },
    handleMax: function() {
        this.fireEvent('valuechanged', this.dsid);
        this.markAndAbortRead();
        dSS.util.callScene(this.dsid, dSS.config.scenes.MAX);
        this.valueField.setValue(100);
        this.valueSlider.setValue(100);
        if (this.matrix.shadebladeang === true) {
            this.bladeAngleField.setValue(100);
            this.angleSlider.setValue(100);
            this.handleBladeAngleChanged.defer(300, this,
                    [ this.bladeAngleField, 100, this.previousBladeAngle ]);
        }
    },
    handleMin: function() {
        this.fireEvent('valuechanged', this.dsid);
        this.markAndAbortRead();
        dSS.util.callScene(this.dsid, dSS.config.scenes.MIN);
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        if (this.matrix.shadebladeang === true) {
            this.bladeAngleField.setValue(0);
            this.angleSlider.setValue(0);
            this.handleBladeAngleChanged.defer(300, this,
                    [ this.bladeAngleField, 0, this.previousBladeAngle ]);
        }
    },
    handleStop: function() {
        this.fireEvent('valuechanged', this.dsid);
        this.markAndAbortRead();
        dSS.util.callScene(this.dsid, dSS.config.scenes.STOP);
        if (this.matrix.shadeposition === true) {
            this.dirtyValue = false;
            this.readDeviceValue.defer(500, this, [ this.dsid ]);
        }
    },
    handleInc: function() {
        this.fireEvent('valuechanged', this.dsid);
        this.markAndAbortRead();

        if (this.matrix.shadeposition === true) {
            dSS.util.callScene(this.dsid, dSS.config.scenes.INC);
            this.dirtyValue = false;
            this.readDeviceValue.defer(500, this, [ this.dsid ]);
        } else {
            var v = this.valueSlider.getValue() + 5;
            if (v > 100) {
                v = 100;
            }

            if (v != this.valueSlider.getValue()) {
                this.valueSlider.setValue(v);
                this.handleSliderChanged(this.valueSlider, v);
            }
        }
    },
    handleDec: function() {
        this.fireEvent('valuechanged', this.dsid);
        this.markAndAbortRead();

        if (this.matrix.shadeposition === true) {
            dSS.util.callScene(this.dsid, dSS.config.scenes.DEC);
            this.dirtyValue = false;
            this.readDeviceValue.defer(500, this, [ this.dsid ]);
        } else {
            var v = this.valueSlider.getValue() - 5;
            if (v < 0) {
                v = 0;
            }

            if (v != this.valueSlider.getValue()) {
                this.valueSlider.setValue(v);
                this.handleSliderChanged(this.valueSlider, v);
            }
        }
    },
    clearData: function()
    {
        this.killRequest();
        this.disable();
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.bladeAngleField.setValue(0);
        this.angleSlider.setValue(0);
        this.dirtyValue = false;
        this.dirtyAngle = false;
    },
    setData: function(dsid, group, present, matrix) {
        this.killRequest();
        this.dsid = dsid;
        this.matrix = matrix;
        if ((matrix.shadeposition == true) && (matrix.outvalue8 === true)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Unsupported configuration"));
            present = false;
        }

        if (matrix.shadeposition === true) {
            this.maxValue = 0xffff;
        } else {
            this.maxValue = 0xff;
        }

        if (matrix.shadeposition === true) {
            this.valueLabel.setText(dSS._("Position") + ':');
        } else {
            this.valueLabel.setText(dSS._("Value") + ':');
        }

        this.bladeAngleLabel.setVisible(matrix.shadebladeang);
        this.bladeAngleUnit.setVisible(matrix.shadebladeang);
        this.bladeAngleField.setVisible(matrix.shadebladeang);
        this.angleSlider.setVisible(matrix.shadebladeang);
        if ((matrix.shadebladeang === true) && (this.notFirstCall === true)) {
            if (this.getHeight() !== this.EXTRA_HEIGHT) {
                this.setHeight(this.EXTRA_HEIGHT);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT);
            }
        }

        if ((group === dSS.config.groups.SHADOW) ||
            (group === dSS.config.groups.ACCESS) ||
            (group === dSS.config.groups.SECURITY)) {
            this.stopButton.setVisible(true);
        } else {
            this.stopButton.setVisible(false);
        }

        if (group === dSS.config.groups.SHADOW) {
            this.minButton.setText(dSS._("Down"));
            this.maxButton.setText(dSS._("Up"));
        } else {
            this.minButton.setText(dSS._("Min"));
            this.maxButton.setText(dSS._("Max"));
        }
        this.doLayout();
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.bladeAngleField.setValue(0);
        this.angleSlider.setValue(0);
        this.dirtyValue = false;
        this.dirtyAngle = false;
        if ((present === true) && (matrix.outvalue8 || matrix.shadeposition)) {
            this.enable();
            this.readDeviceValue.defer(0, this, [this.dsid]);
        } else {
            this.disable();
        }
        this.notFirstCall = true;
    },
    killRequest: function() {
        if (this.AjaxRequestValue !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestValue);
            this.AjaxRequestValue = undefined;
        }
        if (this.AjaxRequestAngle !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestAngle);
            this.AjaxRequestAngle = undefined;
        }
    },
    killRequestValue: function() {
        if (this.AjaxRequestValue !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestValue);
            this.AjaxRequestValue = undefined;
            this.readBladeAngleValue(this.dsid);
        }
    },
    killRequestAngle: function() {
        if (this.AjaxRequestAngle !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestAngle);
            this.AjaxRequestAngle = undefined;
        }
    },
    readDeviceValue: function(reqdsid)
    {
        this.killRequest();

        if (reqdsid !== this.dsid) {
            return;
        }

        this.fireEvent('loading', true);

        var request = 'getConfigWord';
        var offset = 0x02;

        if (this.maxValue <= 0xff) {
            request = 'getConfig';
            offset = 0;
        }

        this.AjaxRequestValue = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsid: reqdsid
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = undefined;
                if ((this.closed === true) ||
                    (this.dirtyValue === true)  ||
                    (reqdsid !== this.dsid)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device value") +
                            ': ' + jsonData.message);
                    } else if (!isNaN(jsonData.result.value)) {
                        var p = this.valueToPercent(jsonData.result.value);
                        this.valueSlider.setValue(p);
                        this.valueField.setValue(p);
                    }

                    if (this.matrix.shadebladeang === true) {
                        this.readBladeAngleValue(this.dsid);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        jsonData.message);

                    if (this.matrix.shadebladeang === true) {
                        this.readBladeAngleValue(this.dsid);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        err);
                }
            }
        });
    },
    markAndAbortRead: function() {
        this.dirtyValue = true;
        this.fireEvent('loading', false);
        this.killRequestValue();
    },
    markAndAbortReadAngle: function() {
        this.dirtyAngle = true;
        this.fireEvent('loading', false);
        this.killRequestAngle();
    },
    readBladeAngleValue: function(reqdsid) {
        this.killRequestValue();

        if (reqdsid !== this.dsid) {
            return;
        }

        this.fireEvent('loading', true);
        var request = 'getConfig';
        var offset = 0x04;

        this.AjaxRequestAngle = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsid: reqdsid
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = undefined;
                if ((this.closed === true) ||
                    (this.dirtyAngle === true)  ||
                    (reqdsid !== this.dsid)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device blade angle position") +
                            ': ' + jsonData.message);
                    } else if (!isNaN(jsonData.result.value)) {
                        this.previousBladeAngle = Math.round(jsonData.result.value * 100 / 0xff)
                        this.bladeAngleField.setValue(this.previousBladeAngle);
                        this.angleSlider.setValue(this.previousBladeAngle);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        jsonData.message);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        err);
                }
            }
        });
    }
});

Ext.reg('dssdevsettingspanel', dSS.devices.DeviceSettingsPanel);

dSS.devices.DeviceSettingsWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        this.HEIGHT_OFFSET = 130;

        if ((config.devices === undefined) || (config.index === undefined) ||
            (config.devices.length === 0)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("No information for device"));
            return;
        }

        var mainPanel = new dSS.devices.DeviceSettingsRootPanel({
            devices: config.devices,
            index: config.index,
            rowSel: config.rowSel,
            border: false,
            maskElement: config.maskElement
        });

        Ext.apply(this, {
            layout: 'fit',
            padding: 10,
            resizable: false,
            height: 306,
            width: dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth'),
            title: dSS._("Device settings"),
            items: [ mainPanel ]
        });

        dSS.devices.DeviceSettingsWindow.superclass.constructor.apply(this,
                                                                     arguments);

        mainPanel.on('updatetitleicon', function(icon) {
            this.setIconClass(icon);
        },
        this);

        mainPanel.on('close', function() {
            this.close();
        },
        this);

        mainPanel.on('updateshadow', function(height) {
            this.setHeight(height + this.HEIGHT_OFFSET)
            var s = this.getEl().shadow;
            if (s) {
                s.realign(this.x, this.y, this.getWidth(), this.getHeight());
            }
        },
        this);

    }
});

// parameters:
//  maskElement
//  device record from the device store
dSS.devices.DeviceSettingsRootPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {

        if (config.rowSel !== undefined) {
            this.rowSel = config.rowSel;
        }

        var disableLeft = true;
        var disableRight = true;

        this.devices = config.devices;
        this.index = config.index;

        var device;
        if (this.devices.length > 0) {
            device = this.devices[this.index];
        }

        if (this.devices.length > 1) {
            if (this.index > 0) {
                disableLeft = false;
            }
            if (this.index < this.devices.length - 1) {
                disableRight = false;
            }
        }

        this.embedded = false;
        if (config.embedded === true) {
            this.embedded = true;
        }

        var devCls;
        if (device !== undefined) {
            var devInfo = dSS.util.parseProductID(device.get('productID'));
            var devCls = dSS.util.parseFunctionID(device.get('functionID'));
            var matrix;
            try {
                matrix = dSS.devprops.GetVisibleProperties(devCls[0],
                                                           devInfo[0],
                                                           devInfo[1]);
            } catch (error) {
                matrix = error;
            }
        } else {
            try {
                matrix = dSS.devprops.GetVisibleProperties(-1, -1, -1);
            } catch (error) {
                matrix = error;
            }
        }

        this.left = new Ext.Button({
            iconCls: 'buttonLeftIcon',
            tooltip: dSS._("Select previous device"),
            width: 50,
            disabled: disableLeft,
            handler: this.previousDevice.createDelegate(this)
        });

        this.right = new Ext.Button({
            iconCls: 'buttonRightIcon',
            tooltip: dSS._("Select next device"),
            width: 50,
            disabled: disableRight,
            handler: this.nextDevice.createDelegate(this)
        });

        this.titleIcon = '';
        var enabled = false;
        var devName = " ";
        var devdSID = "";
        var devPresent = false;
        var devClass = 0;

        if (device !== undefined) {
            this.titleIcon = 'titleIcon' + dSS.util.genCSSName(device);
            enabled = ((device.get('isPresent') === true) &&
                       (matrix.outvalue8 || matrix.shadeposition));
            devName = device.get('name');
            devdSID = device.get('dsid');
            devPresent = device.get('isPresent');
            devClass = devCls[0];
        }

        var buttons =
        [
            this.left,
            this.right,
            new Ext.Toolbar.Fill()
        ];

        var bodyCss;
        var items = [];
        if (this.embedded === false) {
            buttons.push({
                xtype: 'button',
                text: dSS._("Close"),
                scope: this,
                handler: function() {
                    if (this.maskElement !== undefined) {
                        this.maskElement.unmask();
                    }
                    this.fireEvent('close');
                }
            });
            var info =
            {
                xtype: 'panel',
                ref: 'info',
                border: false,
                items:
                [
                    dSS.util.CreateDefaultDeviceInfoPanel(devName, devdSID)
                ]
            }
           
            items.push(info);
        } else {
            bodyCss = 'x-panel-no-left-border x-panel-no-right-border x-panel-no-top-border';
        }

        items.push(
        {
            xtype: 'dssdevsettingspanel',
            ref: 'settings',
            bodyCssClass: bodyCss,
            dsid: devdSID,
            group: devClass,
            present: devPresent,
            matrix: matrix
        });

        Ext.apply(this, {
            bubbleEvents: [ 'updatetitleicon', 'close', 'updateshadow',
                            'valuechanged' ],
            layout: 'table',
            bodyBorder: false,
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            items: items,
            buttonAlign: 'left',
            buttons: buttons
        });

        dSS.devices.DeviceSettingsRootPanel.superclass.constructor.apply(this,
                                                                     arguments);
        this.on('afterrender', function() {
            var icon = enabled ? 'loading-indicator-small' : this.titleIcon;
            this.fireEvent('updatetitleicon', icon);
        },
        this);

        this.addEvents('updateshadow', 'updatetitleicon', 'close');

        if (this.maskElement !== undefined) {
            this.maskElement.mask();
        }

        this.on('beforedestroy', function(component) {
            if (this.maskElement !== undefined) {
                this.maskElement.unmask();
            }
        },
        this);

        this.settings.on('loading', function(l) {
            var icon;
            if (l === true) {
                icon = 'loading-indicator-small';
            } else {
                icon = this.titleIcon;
            }
            this.fireEvent('updatetitleicon', icon);
        },
        this);

    },
    setupButtons: function() {
        if ((this.devices.length > 1) &&
            (this.index < (this.devices.length - 1))) {
            this.right.enable();
        }
        if ((this.devices.length > 1) && (this.index > 0)) {
            this.left.enable();
        }
    },
    loadDevices: function(devices) {
        this.devices = devices;
        this.right.disable();
        this.left.disable();
        if (this.devices.length > 0) {
            this.index = 0;
            //this.setupButtons();
        }
    },
    setRowSelectionModel: function(model) {
        if (this.rowModel !== undefined) {
            this.rowModel.removeListener('rowselect',
                    this.loadRow.createDelegate(this), this);
        }
        this.rowSel = model;
        this.rowSel.addListener('rowselect',
                this.loadRow.createDelegate(this), this);
    },
    clearDialog: function(model, index, record) {
        if (this.ownerCt.isVisible()) {
            this.settings.clearData();
            if (!this.embedded) {
                this.info.removeAll(true);
                this.info.add(dSS.util.CreateDefaultDeviceInfoPanel(" ", " "));
                this.info.doLayout();
            }
        }
    },
    loadRow: function(model, index, record) {
        if (this.ownerCt.isVisible()) {
            if (record.get('isPresent') === false) {
                return;
            }
            if (this.index < this.devices.length) {
                this.index = index;
            }
            this.setupButtons();
            this.updateView(false);
        }
    },
    updateView: function(doSelect) {
        var device = this.devices[this.index];
        var devInfo = dSS.util.parseProductID(device.get('productID'));
        var devCls = dSS.util.parseFunctionID(device.get('functionID'));
        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(devCls[0], devInfo[0],
                                                       devInfo[1]);
        } catch (error) {
            matrix = error;
        }

        this.titleIcon = 'titleIcon' + dSS.util.genCSSName(device);
        this.fireEvent('updatetitleicon', this.titleIcon);

        if (!this.embedded) {
            this.info.removeAll(true);
            this.info.add(
                dSS.util.CreateDefaultDeviceInfoPanel(device.get('name'),
                                                      device.get('dsid')));
            this.info.doLayout();
        }
        this.settings.setData(device.get('dsid'), devCls[0],
                              device.get('isPresent'), matrix);
        if ((this.rowSel !== undefined) && (doSelect !== false)) {
            this.rowSel.selectRow(this.index, false, false);
        }
    },
    previousDevice: function() {
        if (this.index > 0) {
            this.index--;
            if (this.index == 0) {
                this.left.disable();
            }

            if ((this.devices.length > 1) &&
                (this.index < (this.devices.length - 1))) {
                this.right.enable();
            }
        }
        this.updateView();
    },
    nextDevice: function() {
        if (this.index < (this.devices.length - 1)) {
            this.index++;
            if (this.index === (this.devices.length - 1)) {
                this.right.disable();
            }

            if ((this.devices.length > 1) && (this.index > 0)) {
                this.left.enable();
            }
        }
        this.updateView();
    }
});

Ext.reg('dssdevicesettings', dSS.devices.DeviceSettingsRootPanel);
