//= require <dSS/data/Loader>
//= require <dSS/util/Util>
//= require <dSS/util/DefaultDialog>
//= require <dSS/devices/DevicePropertyLoader>
//= require <dSS/devices/DeviceProperties>
//= require <dSS/devices/DeviceSettings>
//= require <dSS/devices/ConsumptionEventForm>
//= require <dSS/devices/AKMSensorConfig>
//= require <dSS/devices/AKMDelayConfig>
//= require <dSS/devices/AKMInputConfig>

Ext.namespace('dSS', 'dSS.grid', 'dSA', "dSS.gthelper");

// custom parameter "featureConfig"
dSS.grid.DevicePanel = Ext.extend(Ext.grid.GridPanel, {
    constructor: function(config) {
        this.featureConfig = config.featureConfig;
        this.areaSceneSelected = false;
        if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
            this.removeFromGroupAllowed = false;
        }

        this.allowDeviceValueChange = true;
        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            this.allowDeviceValueChange = false;
        }

        var commentRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var comment = record.get('comment');

            if ((comment !== undefined) && (comment.length > 0)) {
                return String.format('<div ext:qtip="{0}">{1}</div>',
                        comment, comment);
            } else {
                return "";
            }
        };

        var outputModeRenderer = function(value, metaData, record, rowIndex,
                                          colIndex, store) {
            var functionId = record.get('functionID');
            var mode = record.get('outputMode');
            var val = "";

            if (dSS.util.deviceHasOutput(functionId)) {
                if (mode !== undefined) {
                    switch (mode) {
                        case dSS.config.deviceParams.OUTPUT_MODE_DISABLED:
                            val = dSS._("disabled");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_DIMMER:
                            val = dSS._("dimmed");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_POWERSAVE:
                            val = dSS._("Powersave");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_WIPE:
                            val = dSS._("Wipe");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_ALTSWITCH:
                            val = dSS._("Switch");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH:
                            val = dSS._("switched");
                            break;
                        default:
                            val = "";
                            break;
                    }
                }
            }

            return val;
        };

        var pbRenderer = function(value, metaData, record, rowIndex, colIndex,
                                  store) {
            var buttonID = record.get('buttonID');
            var val = "";
            var productID = record.get('productID');
            var pid = dSS.util.parseProductID(productID);
            var devType = dSS.config.deviceTypes[pid[0]];

            if (buttonID !== undefined) {
                if (record.get('productRevision') < dSS.config.PushbuttonMinFW){
                    return val;
                }

                var functionID = record.get('functionID');
                var fid = dSS.util.parseFunctionID(functionID);
                var color = dSS.config.deviceColors[fid[0]];
                if ((color === 'GN') || (color === 'RT')) {
                    var devNumber = pid[1];

                    if (color === 'GN') {
                        if ((devType === 'TKM') && (devNumber === 200))
                        {
                            return dSS._("leaving / coming home");
                        }

                        if (((devType === 'TKM') && (devNumber === 210)) ||
                            ((devType === 'KM') && (devNumber === 200)) ||
                            ((devType === 'SDS') && (devNumber === 200))) {
                            return dSS._("door bell");
                        }
                    } else if (color == 'RT') {
                        if (((devType === 'TKM') || (devType === 'SDM') ||
                            (devType === 'SDS')) && (devNumber === 200)) {
                            return dSS._("panic");
                        }
                    }
                }

                var activeGroup = record.get('buttonActiveGroup');
                if (activeGroup >= dSS.config.UserGroupStartIndex) {
                    var groupStore = dSS.data.Loader.getGroupStore();
                    var grec = groupStore.getById(activeGroup);
                    if (grec) {
                        return (grec.get('name'));
                    }
                }

                if (devType === 'AKM') {
                    var inputs = record.get('binaryInputs');
                    if (inputs.length < 1) {
                        return "";
                    }
                    var input = inputs[0];
                    if (input.inputType !== undefined) {
                        val = dSS.config.AKMInputTypeStrings[input.inputType];
                        if (val !== undefined) {
                            return dSS._(val);
                        } else {
                            return "";
                        }
                    }
                }

                var c;
                if (color === 'SW') {
                    c = dSS.util.getJokerGroup(record.get('groups'));
                    // assume it's not yet configured
                    if (c === null) {
                        c = dSS.config.groups.JOKER;
                    }
                }

                val = dSS.util.PushButtonToString(buttonID, (c === dSS.config.groups.JOKER));
            }

            return val;
        };

        var buttonInputRenderer = function(value, metaData, record, rowIndex, colIndex,
                                  store) {
            var buttonMode = record.get('buttonInputMode');
            var buttonIndex = record.get('buttonInputIndex');
            var buttonCount = record.get('buttonInputCount');
            var val = "";
            if (buttonMode !== undefined) {
                var fid = dSS.util.parseFunctionID(record.get('functionID'));
                var jokerGroup;
                if (fid[0] === dSS.config.groups.JOKER) {
                    jokerGroup = dSS.util.getJokerGroup(record.get('groups'));
                }
                var productID = record.get('productID');
                var pid = dSS.util.parseProductID(productID);
                var devType = dSS.config.deviceTypes[pid[0]];
                if (devType === 'AKM') {
                    val = dSS.config.AKMButtonInputStrings[
                        record.get('AKMInputProperty')];
                } else {
                    val = dSS.util.PushButtonModeToString(buttonMode,
                                                      buttonIndex,
                                                      buttonCount, jokerGroup,
                                                      (devType === 'SDS'));
                }
            }

            return val;
        };

        var dSIDdeviceRenderer= function(value, metaData, record, rowIndex, colIndex, store) {
            var shortId = dSS.util.shortDSID(record.get('dsid'));
            if (record.get('isValid') === false) {
                shortId = dSS._('Loading') + ' ... ' + shortId;
            }
            return String.format('<div ext:qtip="{0}">{1}</div>',
                                  record.get('dsid'), shortId);
        };

        var dSIDmeterRenderer= function(value, metaData, record, rowIndex, colIndex, store) {
            var shortId = dSS.util.shortDSID(record.get('meterDSID'));
            return String.format('<div ext:qtip="{0}">{1}</div>',
                                  record.get('meterDSID'), shortId);
        };

        var getEANString = function(record) {
            var eanString;

            if (record.get('OemStatus') !== 'Valid') {
                return eanString;
            }

            var serial = record.get('OemSerialNumber');
            var eanName = record.get('HW');
            if ((eanName !== undefined) && (eanName.length > 0)) {
                eanString = eanName + '<br/>';
            }
            var ean = record.get('OemEanNumber');
            if ((ean !== undefined) && (ean.length > 0)) {
                eanString += dSS._("EAN") + ": " + ean + '<br/>';
            }
            eanString += dSS._("Serial number") + ": " +
                record.get('OemSerialNumber');
            return eanString;
        };

        var devTypeRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var present = record.get('isPresent');

            var retval = String.format('<img class="padding-img" src="{0}"/>', Ext.BLANK_IMAGE_URL);
            var eanString = getEANString(record);
            if ((eanString !== undefined) && (eanString.length > 0)) {
                retval = String.format('<div ext:qtip="{0}">{1}&nbsp;&nbsp;</div>',
                                        eanString, retval);
            }
            if (record.get('isValid') === false) {
                metaData.css = "deviceInfoLoading";
            } else {
                metaData.css = "gridIcon" + dSS.util.genCSSName(record);
            }
            return retval;
        };

        var devNameRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var eanString = getEANString(record);
            var name = record.get('name') ? record.get('name') : '&nbsp;&nbsp;&nbsp;&nbsp;';
            if ((eanString !== undefined) && (eanString.length > 0)) {
                return String.format('<div ext:qtip="{0}">{1}</div>',
                                     eanString, name);
            } else {
                return value;
            }
        };

        var zoneNameRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            return value ? value: dSS._("Zone") + ' #' + record.get('zoneId');
        };

        var defaultDateRenderer = Ext.util.Format.dateRenderer('Y-m-d\\&\\n\\b\\s\\p\\;H:i:s');

        var inactiveSinceRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var d = record.get('inactiveSince');
            if ((d == undefined) || (d.getFullYear() == 1970)) {
                return "";
            } else {
                return defaultDateRenderer(d);
            }
        };

        var userGroupRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var deviceGroups = record.get('groups');
            var groupNames = "";
            var addBreak = false;
            var groupStore = dSS.data.Loader.getGroupStore();
            Ext.each(deviceGroups, function(group) {
                if (group < dSS.config.UserGroupStartIndex) {
                    return true;
                }
                var gr = groupStore.getById(group);
                if (gr) {
                    var gn = gr.get('name');
                    if (gn.length > 0) {
                        if (addBreak) {
                            groupNames = groupNames + '<br/>';
                        }
                        groupNames = groupNames + gn;
                        addBreak = true;
                    }
                }
            });
            return groupNames;
        };

        var hideColumn = false;
        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            hideColumn = true;
        }

        var deviceCols = [
        {
            header: "",
            width: 20,
            resizable: false,
            sortable: false,
            hideable: false,
            menuDisabled: true,
            fixed: true,
            dataIndex: 'functionID',
            renderer: devTypeRenderer,
            id: 'functionID'
        },
        {
            header: dSS._("Name"),
            tooltip: dSS._("Name"),
            width: 80,
            sortable: true,
            dataIndex: 'name',
            renderer: devNameRenderer,
            id: 'name'
        },
        {
            header: dSS._("dSID"),
            tooltip: dSS._("dSID"),
            width: 70,
            sortable: true,
            dataIndex: 'dsid',
            id: 'dsid',
            renderer: dSIDdeviceRenderer
        }];

        deviceCols.push(
            {
                header: dSS._("Output mode"),
                tooltip: dSS._("Output mode"),
                sortable: true,
                dataIndex: 'outputMode',
                id: 'outputMode',
                renderer: outputModeRenderer
            });

        deviceCols.push(
            {
                header: dSS._("Button mode"),
                tooltip: dSS._("Button mode"),
                sortable: true,
                dataIndex: 'buttonID',
                id: 'buttonID',
                renderer: pbRenderer
            });

        if (this.featureConfig !== dSS.config.FeatureConfig.GROUP) {
            deviceCols.push(
                {
                    header: dSS._("Button input"),
                    tooltip: dSS._("Button input"),
                    hidden: hideColumn,
                    sortable: true,
                    dataIndex: 'buttonMode',
                    id: 'buttonMode',
                    renderer: buttonInputRenderer
                });
        } else {
            deviceCols.push(
                {
                    header: dSS._("User groups"),
                    tooltip: dSS._("User groups"),
                    hidden: hideColumn,
                    sortable: true,
                    dataIndex: 'groups',
                    id: 'userGroups',
                    renderer: userGroupRenderer
                });
        }

        if (this.featureConfig !== dSS.config.FeatureConfig.CIRCUIT) {
            deviceCols.push(
            {
                header: dSS._("dSM Name"),
                tooltip: dSS._("dSM Name"),
                width: 80,
                sortable: true,
                dataIndex: 'meterName',
                id: 'meterName'
            });
        }

        deviceCols.push(
        {
            header: dSS._("Room name"),
            tooltip: dSS._("Room name"),
            hidden: hideColumn,
            width: 80,
            sortable: true,
            dataIndex: 'zoneName',
            id: 'zoneName',
            renderer: zoneNameRenderer
        });

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            deviceCols = deviceCols.concat(
            [
                {
                    header: dSS._("Room ID"),
                    tooltip: dSS._("Room ID"),
                    width: 35,
                    sortable: true,
                    dataIndex: 'zoneId',
                    id: 'zoneId'
                },
                {
                    header: dSS._("HW info"),
                    tooltip: dSS._("Hardware info"),
                    width: 50,
                    sortable: true,
                    id: 'HW',
                    dataIndex: 'HW'
                },
                {
                    header: dSS._("Firmware version"),
                    tooltip: dSS._("Firmware version"),
                    width: 50,
                    sortable: true,
                    id: 'fw',
                    dataIndex: 'firmwareVersion'
                },
                {
                    header: dSS._("Registered on"),
                    tooltip: dSS._("Registered on"),
                    width: 60,
                    sortable: true,
                    dataIndex: 'firstSeen',
                    xtype: 'datecolumn',
                    format: 'Y-m-d\\&\\n\\b\\s\\p\\;H:i:s',
                    id: 'firstSeen'
                },
                {
                    header: dSS._("Inactive since"),
                    tooltip: dSS._("Inactive since"),
                    width: 60,
                    sortable: true,
                    dataIndex: 'inactiveSince',
                    renderer: inactiveSinceRenderer,
                    id: 'inactiveSince'
                }
            ]);
        }

        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            deviceCols.push(
                {
                    header: dSS._("Comment"),
                    tooltip: dSS._("Comment"),
                    hidden: hideColumn,
                    width: 20,
                    resizable: false,
                    sortable: false,
                    dataIndex: 'comment',
                    id: 'comment',
                    renderer: commentRenderer
                });
        }

        var selModel;

        if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            selModel = new Ext.grid.RowSelectionModel({ singleSelect: true });
        } else {
            selModel = new Ext.grid.RowSelectionModel({ singleSelect: false });
        }

        var enabledd = false;
        var ddgroup = "";
        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            enabledd = true;
            ddgroup = "zoneDeviceDD";
        } else if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
            enabledd = true;
            ddgroup = "groupDeviceDD";
        }

        Ext.apply(this, {
            store: dSS.data.Loader.getDeviceStore(),
            columns: deviceCols,
            ddGroup: ddgroup,
            bubbleEvents: [ 'removedfromgroup' ],
            enableDragDrop: enabledd,
            stripeRows: true,
            title: (this.featureConfig === dSS.config.FeatureConfig.SCENE) ? undefined :  dSS._("Device", "Devices", 2),
            selModel: selModel,
            viewConfig: {
                forceFit: true,
                getRowClass: function(record, index) {
                    var c = record.get('isPresent');
                    if (c === false) {
                        return 'nonPresentDevice';
                    }
                    return '';
                }
            }
        });

        this.tbar = this.buildTopToolbar();

        dSS.grid.DevicePanel.superclass.constructor.apply(this, arguments);

        this.addEvents('removedfromgroup');

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            dSS.MessageBus.on('setdevicename', this.setDeviceName.createDelegate(this),
            this);
        }

        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            dSS.MessageBus.on('sceneselected', this.handleSceneSelect.createDelegate(this),
            this);
        }


        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            this.on('afterrender', function() {
                    var gv = this.getView();
                    this.origb4Drag = gv.dragZone.b4Drag.createDelegate(gv.dragZone);

                    gv.dragZone.b4Drag = function(e) {
                        var store = this.grid.getStore();
                        var sel = this.grid.getSelectionModel();
                        var selected = sel.getSelections();
                        var i;
                        for (i = 0; i < selected.length; i++) {
                            var record = selected[i];
                            if (record.get('isPresent') === false) {
                                var index = store.findExact('dsid',
                                        record.get('dsid'));
                                if (index > -1) {
                                    sel.deselectRow(index, false);
                                }
                            }
                        }
                        selected = sel.getSelections();
                        if (selected.length > 0) {
                            var proxy = this.getProxy();
                            var ghost = proxy.getGhost();
                            ghost.update(dSS.gt.strargs(
                                            dSS.gt.ngettext(
                                                "%1 device selected",
                                                "%1 devices selected",
                                                selected.length),
                                            [ selected.length ]));
                            this.grid.origb4Drag(e);
                        }
                    };
                    gv.dragZone.onBeforeDrag = function(data, evt) {
                        var i;
                        var retval = false;
                        for (i = 0; i < data.selections.length; i++) {
                            var record = data.selections[i];
                            if (record.get('isPresent') === true) {
                                retval = true;
                            }
                        }
                        return retval;
                    };
                },
                this);
        }


        if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
            this.getSelectionModel().on('beforerowselect',
                    function(sel, rowIndex, keepExisting, record) {
                        // no extra checks needed for single select
                        if (keepExisting === false) {
                            return true;
                        }

                        // do not allow mixed color multiple selections
                        var previous = sel.getSelected();
                        if (previous === undefined) {
                            return true;
                        }


                        // AKMs have no color
                        var pid = dSS.util.parseProductID(
                                                record.get('productID'));
                        if (pid[0] === dSS.config.deviceTypeNum.AKM) {
                            return true;
                        }

                        pid = dSS.util.parseProductID(
                                                previous.get('productID'));
                        // if previous selection is an AKM also we have to
                        // check the whole range, first non AKM will define
                        // the colora
                        var nonAKM = false;
                        if (pid[0] === dSS.config.deviceTypeNum.AKM) {
                            var range = sel.getSelections();
                            Ext.each(range, function(rec) {
                                pid = dSS.util.parseProductID(
                                                        rec.get('productID'));
                                if (pid[0] !== dSS.config.deviceTypeNum.AKM) {
                                    nonAKM = true;
                                    previous = rec;
                                    // abort loop
                                    return false;
                                }
                            });
                        } else {
                            nonAKM = true;
                        }

                        if (nonAKM !== true) {
                            return true;
                        }

                        // for all others the color has to match
                        previouscolor = dSS.util.getDeviceColor(previous);
                        var currentcolor = dSS.util.getDeviceColor(record);
                        if (previouscolor !== currentcolor) {
                            return false;
                        }

                        return true;
                },
                this);

            this.on('groupselection', function(group, state) {
                    this.getTopToolbar().removeFromGroupButton.setDisabled(state);
                    this.removeFromGroupAllowed = state;
                    this.selectedGroup = group;
                },
                this);
        }
        this.on('rowclick', this.handleDeviceRowSelect.createDelegate(this), this);

        dSS.MessageBus.on('structureloaded',
                this.handleSelection.createDelegate(this), this);

        // Here you can add functionality that requires the object to
        // exist, like event handling.
        this.on('rowcontextmenu', function(grid, rowIndex, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var sel = this.getSelectionModel();
            if (!sel.isSelected(rowIndex)) {
                sel.selectRow(rowIndex, false);
                this.handleDeviceRowSelect(grid, rowIndex, event);
            }

            var menuItem;
            var showRemoveMenuItem = false;
            var presentInSelection = false;
            var selectedDeviceIsPresent = false;
            var deviceHasOutput = false;
            var showRemoveFromGroupMenuItem = false;

            if (sel.getSelected().get("isPresent") === true) {
                selectedDeviceIsPresent = true;
            }

            if (dSS.util.deviceHasOutput(sel.getSelected().get("functionID")) === true) {
                deviceHasOutput = true;
            }

            var records = sel.getSelections();

            var i;
            for (i = 0; i < records.length; i++) {
                var record = records[i];

                // if at least one record can be removed - set the flag
                if (record.get('isPresent') === false) {
                    showRemoveMenuItem = true;
                } else {
                    presentInSelection = true;
                    showRemoveFromGroupMenuItem = true;
                }
            }

            menuItem = new Ext.menu.Item({
                text: dSS._("Blink"),
                iconCls: 'blink',
                handler: this.blinkDevice.createDelegate(this, rowIndex,
                                                         true),
                disabled : !presentInSelection
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Edit device values"),
                iconCls: 'editValueToolbarIcon',
                handler: this.showControlDeviceWindow.createDelegate(
                                                this, rowIndex, true),
                disabled: !selectedDeviceIsPresent,
                hidden: !this.allowDeviceValueChange
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Add to area"),
                    iconCls: 'addToAreaIcon',
                    handler: this.addToArea.createDelegate(this, rowIndex, true),
                    disabled: !this.areaSceneSelected || !deviceHasOutput
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    text: dSS._("Remove from area"),
                    iconCls: 'removeFromAreaIcon',
                    handler: this.removeFromArea.createDelegate(this, rowIndex, true),
                    disabled: !this.areaSceneSelected || !deviceHasOutput
                });
                this.contextMenu.add(menuItem);

                this.contextMenu.add('-');
            }

            if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Test transmission quality"),
                    iconCls: 'transmissionQuality',
                    handler: this.getTransmissionQuality.createDelegate(
                                                    this, rowIndex, true),
                    disabled: !selectedDeviceIsPresent
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    text: dSS._("Update device status"),
                    iconCls: 'checkDevicePresenceIcon',
                    handler: this.getDevicePresence.createDelegate(
                                                    this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);

                this.contextMenu.add('-');
            }

            if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
                (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Edit device properties"),
                    iconCls: (this.featureConfig === dSS.config.FeatureConfig.SCENE) ? 'sceneDevicePropertiesIcon' : 'devicePropertiesIcon',
                    handler: this.showDevicePropertiesWindow.createDelegate(
                                                    this, rowIndex, true),
                    disabled: !selectedDeviceIsPresent
                });
                this.contextMenu.add(menuItem);
            }

            if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
                var dr = sel.getSelected();
                var devInfo = dSS.util.parseProductID(dr.get('productID'));
                var type = devInfo[0];
                var number = devInfo[1];
                var cls = dSS.util.parseFunctionID(record.get('functionID'));
                var matrix;

                try {
                    matrix = dSS.devprops.GetVisibleProperties(cls[0], type,
                                                               number);
                } catch (error) {
                    // for now we can safely assume all as invisible
                    matrix = error;
                }

                menuItem = new Ext.menu.Item({
                    text: dSS._("Edit shade device properties"),
                    iconCls: 'shadePropertiesIcon',
                    handler: this.showShadePropertiesWindow.createDelegate(
                                                    this, rowIndex, true),
                    disabled: !(selectedDeviceIsPresent && matrix.shadeprops)
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    text: dSS._("Configure consumption event"),
                    iconCls: 'consumptionEventIcon',
                    handler: this.showConsumptionThresholdWindow.createDelegate(
                                                    this, rowIndex, true),
                    disabled: !(selectedDeviceIsPresent && matrix.consumption)
                });
                this.contextMenu.add(menuItem);
            }

            menuItem = new Ext.menu.Item({
                text: dSS._("Rename device"),
                iconCls: 'editIcon',
                handler: this.editDevice.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
                (this.featureConfig === dSS.config.FeatureConfig.GROUP) ||
                (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Edit comment"),
                    iconCls: 'editCommentIcon',
                    handler: this.editComment.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
                (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {

                this.contextMenu.add('-');

                menuItem = new Ext.menu.Item({
                    text: dSS._("Remove device"),
                    iconCls: 'removeDeviceIcon',
                    handler: this.removeDevice.createDelegate(this, rowIndex,
                                                              true),
                    disabled: !showRemoveMenuItem
                });
                this.contextMenu.add(menuItem);
            }


            if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
                this.contextMenu.add('-');

                if (this.removeFromGroupAllowed !== true) {
                    showRemoveFromGroupMenuItem = false;
                }
                menuItem = new Ext.menu.Item({
                    text: dSS._("Remove from group"),
                    iconCls: 'removeDeviceIcon',
                    handler: this.removeDeviceFromGroup.createDelegate(
                                                        this, rowIndex, true),
                    disabled: showRemoveFromGroupMenuItem
                });
                this.contextMenu.add(menuItem);
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },

    handleSelection: function() {
        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            return;
        }

        if (!this.ownerCt.isVisible()) {
            return;
        }

        if (this.getStore().getCount() === 0) {
            return;
        }

        var selModel = this.getSelectionModel();
        var record;
        var index;

        if (selModel.hasSelection()) {
            record = selModel.getSelected();
            index = this.getStore().indexOf(record);
        } else if (this.lastSelectedDSID !== undefined) {
            index = this.getStore().findExact('dsid', this.lastSelectedDSID);
            if (index != -1) {
                record = this.getStore().getAt(index);
            }
        }

        if (record !== undefined) {
            this.lastSelectedDSID = record.get('dsid');
            this.getView().focusRow(index);
            selModel.selectRecords([record], false);
        }
    },
    setDeviceValueButtonVisibility: function(show) {
        if (show === true) {
            this.getTopToolbar().editValueButton.show();
            this.allowDeviceValueChange = true;
        } else {
            this.getTopToolbar().editValueButton.hide();
            this.allowDeviceValueChange = false;
        }
    },
    buildTopToolbar: function() {
        var bar = [];

        bar.push({
            iconCls: 'blink',
            handler: this.blinkDevice,
            tooltip: dSS._("Blink device"),
            ref: 'blinkButton',
            scope: this
        });

        bar.push({
            iconCls: 'editValueToolbarIcon',
            handler: this.showControlDeviceWindow,
            tooltip: dSS._("Edit device values"),
            ref: 'editValueButton',
            hidden: !this.allowDeviceValueChange,
            scope: this
        });

        bar.push('-');

        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            bar = bar.concat(
            [
                {
                    ref: 'addToAreaButton',
                    iconCls: 'addToAreaIcon',
                    handler: this.addToArea,
                    tooltip: dSS._("Add to area"),
                    scope: this,
                    disabled: true
                },
                {
                    ref: 'removeFromAreaButton',
                    iconCls: 'removeFromAreaIcon',
                    handler: this.removeFromArea,
                    tooltip: dSS._("Remove from area"),
                    scope: this,
                    disabled: true
                },
                '-'
           ]);
        }

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            bar = bar.concat(
            [
                {
                    iconCls: 'transmissionQuality',
                    handler: this.getTransmissionQuality,
                    tooltip: dSS._("Test transmission quality"),
                    ref: 'transmissionQualityButton',
                    scope: this
                },
                {
                    iconCls: 'checkDevicePresenceIcon',
                    handler: this.getDevicePresence,
                    tooltip: dSS._("Update device status"),
                    scope: this
                },
                '-'
           ]);
        }

        if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            bar.push(
                {
                    iconCls: (this.featureConfig === dSS.config.FeatureConfig.SCENE) ? 'sceneDevicePropertiesIcon' : 'devicePropertiesIcon',
                    handler: this.showDevicePropertiesWindow,
                    tooltip: dSS._("Configure device properties"),
                    ref: 'devicePropertiesButton',
                    scope: this
                });
        }

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            bar.push(
                {
                    iconCls: 'shadePropertiesIcon',
                    handler: this.showShadePropertiesWindow,
                    tooltip: dSS._("Edit shade device properties"),
                    ref: 'shadePropertiesButton',
                    scope: this
                },
                {
                    iconCls: 'consumptionEventIcon',
                    handler: this.showConsumptionThresholdWindow,
                    tooltip: dSS._("Configure consumption event"),
                    ref: 'consumptionEventButton',
                    scope: this
                });
        }

        bar.push(
        {
            iconCls: 'editDeviceToolbarIcon',
            handler: this.editDevice,
            tooltip: dSS._("Rename device"),
            scope: this
        });

        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            bar.push(
            {
                iconCls: 'editCommentToolbarIcon',
                handler: this.editComment,
                tooltip: dSS._("Edit comment"),
                scope: this
            });
        }

        if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
            bar.push(
            {
                iconCls: 'removeFromAreaIcon',
                handler: this.removeDeviceFromGroup,
                tooltip: dSS._("Remove device from group"),
                ref: 'removeFromGroupButton',
                scope: this
            });
        }


        bar.push('->');

        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            bar.push(
            {
                iconCls: 'removeDeviceToolbarIcon',
                handler: this.removeDevice,
                tooltip: dSS._("Remove device"),
                ref: 'removeDeviceButton',
                scope: this
            });
        }

        return bar;
    },

    showControlDeviceWindow: function(item, event, rowIndex) {
        var records = this.getStore().getRange();
        var sel = this.getSelectionModel();
        var index;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            var dsid = sel.getSelected().get('dsid');
            var i;
            for (i = 0; i < records.length; i++) {
                if (records[i].get('dsid') === dsid) {
                    index = i;
                    break;
                }
            }
        } else {
            index = rowIndex;
        }

        var win = new dSS.devices.DeviceSettingsWindow({
            devices: records,
            index: index,
            rowSel: sel,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show(this);
    },
    setDeviceName: function(dsid, newName) {

        var record = this.store.getById(dsid);

        var oldName;
        if (record !== undefined)
        {
            oldName = record.get('name');
        }

        dSS.util.genericRequest({
                url: '/json/device/setName',
                params: {
                    dsid: dsid,
                    newName: newName
                },
                failureMsg: dSS._("Could not rename device"),
                scope: this
            },
            function(success) {
                if (success === true) {
                    if (record !== undefined) {
                        record.set('name', newName);
                        record.commit();
                        // we are only interested in name changes
                        // from nothing to something or vice versa
                        //
                        // check that the names were not equal is done
                        // before submitting the ajax request
                        if ((oldName === undefined) ||
                            (oldName.length === 0) ||
                            (newName === undefined) ||
                            (newName.length === 0)) {
                            dSS.data.Loader.getZoneStore().fireEvent(
                                    'zonedevnamechanged', record);
                        }
                    } else {
                        dSS.MessageBus.fireEvent('refreshview');
                    }
                    dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Device") + ' &quot;' +
                            dSS.util.getName(newName) +
                            '&quot; (' + dSS.util.shortDSID(dsid) +
                            ') ' + dSS._("was renamed"));
                }
            });
    },
    handleSceneSelect: function(scene) {
        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            if (dSS.util.isAreaScene(scene)) {
                this.areaSceneSelected = true;
                var sel = this.getSelectionModel();
                if (sel.hasSelection()) {
                    var record = sel.getSelected();
                    if (dSS.util.deviceHasOutput(record.get('functionID'))) {
                        this.getTopToolbar().addToAreaButton.enable();
                        this.getTopToolbar().removeFromAreaButton.enable();
                        return;
                    }
                }

            } else {
                this.areaSceneSelected = false;
            }
            this.getTopToolbar().addToAreaButton.disable();
            this.getTopToolbar().removeFromAreaButton.disable();
        }
    },
    handleDeviceRowSelect: function(grid, rowIndex, event) {
        var sel = this.getSelectionModel();
        var tb = this.getTopToolbar();

        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            if ((dSS.util.deviceHasOutput(sel.getSelected().get("functionID")) === true) &&
                (this.areaSceneSelected === true)) {
                tb.addToAreaButton.enable();
                tb.removeFromAreaButton.enable();
            } else {
                tb.addToAreaButton.disable();
                tb.removeFromAreaButton.disable();
            }
        } else {
            this.lastSelectedDSID = sel.getSelected().get("dsid");
        }


        var present = sel.getSelected().get("isPresent");
        this.getTopToolbar().blinkButton.setDisabled(!present);
        var b = this.getTopToolbar().editValueButton;
        if (b !== undefined) {
            b.setDisabled(!present);
        }
        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            tb.removeDeviceButton.setDisabled(present);
        }
        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            tb.transmissionQualityButton.setDisabled(!present);
            tb.devicePropertiesButton.setDisabled(!present);
            tb.consumptionEventButton.setDisabled(!present);

            var rec = sel.getSelected();
            var devInfo = dSS.util.parseProductID(rec.get('productID'));
            var type = devInfo[0];
            var number = devInfo[1];
            var cls = dSS.util.parseFunctionID(rec.get('functionID'));
            var matrix;
            try {
                matrix = dSS.devprops.GetVisibleProperties(cls[0], type, number);
            } catch (error) {
                // for now we can safely assume all as invisible
                matrix = error;
            }

            if (present === true) {
                tb.shadePropertiesButton.setDisabled(!matrix.shadeprops);
                tb.consumptionEventButton.setDisabled(!matrix.consumption);
            } else {
                tb.shadePropertiesButton.disable();
                tb.consumptionEventButton.disable();
            }
        }
    },
    editDevice: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }
            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        Ext.Msg.prompt(dSS._("Rename device"), dSS._("Name for") + ' ' +
                       dSS.util.shortDSID(record.get('dsid')) + ': ',
        function(btn, text) {
            if ((btn === 'ok') && (text !== record.get('name'))) {
                this.setDeviceName(record.get('dsid'), text);
            }
        },
        this, false, record.get('name'));
    },
    addToArea: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }
            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var scene;
        try {
            scene = this.findParentByType('dssscenespanel').getSelectedScene();
        } catch (err) {
            Ext.Msg.alert(dSS._("Error"), err);
            return;
        }

        if (!dSS.util.isAreaScene(scene)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device can only be added to area scenes!"));
            return;
        }

        dSS.util.genericRequest({
                url: '/json/device/addToArea',
                params: {
                    dsid: record.get('dsid'),
                    areaScene: scene
                },
                failureMsg: dSS._("Could not add device to area"),
                scope: this,
                maskEl: this.findParentByType(Ext.Viewport).el,
                maskMsg: dSS.gt.strargs(
                             dSS._("Adding device %1 to %2"),
                             [
                                 '&quot;' +
                                 dSS.util.getNameOrDSID(record.get('name'),
                                                        record.get('dsid')) +
                                 '&quot;',
                                 dSS._(dSS.config.sceneNamesMsg[scene])
                             ]
                         )
            },
            function(success) {
                if (success === true) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS.gt.strargs(
                            dSS._("Device %1 was added to %2"),
                            [
                                '&quot;' +
                                dSS.util.getNameOrDSID(record.get('name'),
                                                       record.get('dsid')) +
                                '&quot;',
                                dSS._(dSS.config.sceneNamesMsg[scene])
                            ]
                        )
                    );
                }
            });
    },
    removeFromArea: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }
            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var scene;
        try {
            scene = this.findParentByType('dssscenespanel').getSelectedScene();
        } catch (err) {
            Ext.Msg.alert(dSS._("Error"), err);
            return;
        }

        if (!dSS.util.isAreaScene(scene)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device can only be removed from area scenes!"));
            return;
        }

        dSS.util.genericRequest({
                url: '/json/device/removeFromArea',
                params: {
                    dsid: record.get('dsid'),
                    areaScene: scene
                },
                failureMsg: dSS._("Could not remove device from area"),
                scope: this,
                maskEl: this.findParentByType(Ext.Viewport).el,
                maskMsg: dSS.gt.strargs(
                            dSS._("Removing device %1 from %2"),
                            [
                                '&quot;' +
                                dSS.util.getNameOrDSID(record.get('name'),
                                                       record.get('dsid')) +
                                '&quot;',
                                dSS._(dSS.config.sceneNamesMsg[scene])
                            ]
                        )
            },
            function(success) {
                if (success === true) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS.gt.strargs(
                            dSS._("Device %1 was removed from %2"),
                            [
                                '&quot;' +
                                dSS.util.getNameOrDSID(record.get('name'),
                                                       record.get('dsid')) +
                                '&quot;',
                                dSS._(dSS.config.sceneNamesMsg[scene])
                            ]
                        )
                    );
                }
            });
    },

    getTransmissionQuality: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: can not request transmission quality from a non present device!"));
            return;
        }

        var name = record.get('name');
        var dsid = record.get('dsid');

        Ext.Ajax.request({
            scope: this,
            url: '/json/device/getTransmissionQuality',
            disableCaching: true,
            method: "GET",
            params: {
                dsid: dsid
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var title = dSS._("Transmission Quality");

                        var up = jsonData.result.upstream * (100/dSS.config.TransmissionQuality.UPSTREAM_MAX);
                        var down = 100 - jsonData.result.downstream * (100/dSS.config.TransmissionQuality.DOWNSTREAM_MAX);
                        up = Math.round(up);
                        down = Math.round(down);

                        var win = new dSS.util.DefaultDialogWindow({
                            title: title,
                            name: name,
                            dsid: dsid,
                            extraItems:
                            [
                                {
                                    xtype: 'dssutildialogcontentpanel',
                                    strings:
                                    [
                                        dSS._("Upstream communication")+':&nbsp;',
                                        up + '%',
                                        dSS._("Downstream communication")+':&nbsp;',
                                        down + '%'
                                    ]
                                }
                            ],
                            maskElement: this.findParentByType(Ext.Viewport).el
                        });
                        win.show();

                    } else {
                        Ext.MessageBox.alert(dSS._("Error"),
                                dSS._("Could not get transmission quality: "),
                                  jsonData.message);
                    }
                }
                catch(err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not get transmission quality") +
                                  ': ' + err);
                }
            },
            failure: function(result, request) {
                var message = '';
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if ((!jsonData.ok) && (jsonData.message.length > 0)) {
                        message = ': ' + jsonData.message;
                    }
                }
                catch(err) {}
                Ext.MessageBox.alert(dSS._("Error"),
                        dSS._("Could not get transmission quality") + message);
            }
        });
    },

    getDevicePresence: function(item, event, rowIndex) {
        if (dSA.module !== true) {
            Ext.Msg.alert(dSS._("Notice"),
                          dSS._("This feature is only available in the " +
                                "setup on dSS11 hardware"));
            return;
        }
        var record;
        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }
            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }
        var name = record.get('name');
        var dsid = record.get('dsid');
        var dsmdsid = record.get('meterDSID');
        var wasPresent = record.get('isPresent');
        dSA.Reactor.fireEvent('checksingledevicepresence',
            dsid, dsmdsid, name, wasPresent,
            this.findParentByType(Ext.Viewport).el);
    },

    removeDevice: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }
        records = sel.getSelections();

        var skippedPresent = false;
        var notPresent = [];

        var i;
        for (i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent') === true) {
                skippedPresent = true;
                continue;
            }

            notPresent.push(record);
        }

        if (notPresent.length > 0) {
            this.deleteDeviceConfirm(notPresent);
        } else {
            if (skippedPresent === true) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Present devices can not be removed!"));
            }
        }
    },

    deleteDeviceConfirm: function(records) {
        var store = this.getStore();
        if (records.length < 1) {
            this.findParentByType(Ext.Viewport).el.unmask();
            return;
        }

        var record = records.shift();

        var win = new dSS.util.DefaultDialogWindow({
            title: dSS._("Delete Device"),
            name: record.get('name'),
            dsid: record.get('dsid'),
            cancelButtonText: dSS._("Cancel"),
            scope: this,
            extraItems:
            [
                {
                    xtype: 'panel',
                    html: '<p>' +
                          dSS._("Are you sure you want to delete this " +
                                "device?") + '</p><p>' +
                          dSS._("Deleting the device will remove its " +
                                "configuration. Therefore, the device should " +
                                "only be deleted if it is permanently " +
                                "removed from this digitalSTROM " +
                                "installation. If you want to connect the " +
                                "device again, you might need to reregister " +
                                "the device manually within the power " +
                                "circuits tab.") +
                          '</p>'
                }
            ],
            maskElement: this.findParentByType(Ext.Viewport).el,
            buttons:
            [
                {
                    xtype: 'button',
                    text: dSS._("Delete"),
                    scope: this,
                    handler: function() {
                        win.close();
                        var f = this.deleteDevice.createDelegate(this,
                                                                 [
                                                                     record,
                                                                     records
                                                                 ]);
                        f();
                    }
                }
            ]
        });
        win.show();
    },

    deleteDevice: function(record, remainingRecords) {
        var maskel = this.findParentByType(Ext.Viewport).el;
        maskel.mask(dSS._("Deleting device") + ": " + record.get("dsid"));
        Ext.Ajax.request({
            url: '/json/structure/removeDevice',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                deviceID: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var dSMStore = dSS.data.Loader.getModulatorStore();
                        var i = dSMStore.find('dsid', record.get('meterDSID'));
                        if (i > -1) {
                            var dsm = dSMStore.getAt(i);
                            dsm.set('numDevices', dsm.get('numDevices') - 1);
                            dsm.commit();
                        }
                        this.getStore().remove(record);
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Device") + ' &quot;' +
                            dSS.util.getName(record.get('name')) + '&quot; (' +
                            dSS.util.shortDSID(record.get('dsid')) +
                            ') ' + dSS._("was removed"));

                        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
                            var zb = this.findParentByType('dsszonebrowser');
                            if (zb) {
                                zb.zonePanel.zoneView.updateNumDevicesHeader();
                            }
                        }
                        this.deleteDeviceConfirm(remainingRecords);
                    } else {
                        maskel.unmask();
                        Ext.MessageBox.alert(dSS._("Could not remove"), jsonData.message);
                    }
                }
                catch(err) {
                    maskel.unmask();
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove device command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                maskel.unmask();
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove device command to dSS"));
            }
        });
    },

    sendBlinkRequest: function(record) {
        record.set('blink', true);
        var unblink = function(record) {
            record.set('blink', false);
        };

        Ext.Ajax.request({
            url: '/json/device/blink',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                dsid: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        Ext.Msg.alert(dSS._("Could not blink"),
                                      jsonData.message);
                    } else {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Device") + ' &quot;' +
                            dSS.util.getName(record.get('name')) + '&quot; (' +
                            dSS.util.shortDSID(record.get('dsid')) +
                              ') ' + dSS._("is blinking now"));
                    }
                } catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send blink device command to dSS") + ": " + err);
                }
                unblink.defer(2000, this, [record]);
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send blink device command to dSS"));
                unblink.defer(2000, this, [record]);
            }
        });
    },

    blinkDevice: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }
        records = sel.getSelections();
        var i;
        for (i = 0; i < records.length; i++) {
            if (records[i].get('isPresent') === false) {
                continue;
            }

            this.sendBlinkRequest(records[i]);
        }
    },
    showShadePropertiesWindow: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var devInfo = dSS.util.parseProductID(record.get('productID'));
        var type = devInfo[0];
        var number = devInfo[1];
        var cls = dSS.util.parseFunctionID(record.get('functionID'));
        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(cls[0], type, number);
        } catch (error) {
            // for now we can safely assume all as invisible
            matrix = error;
        }

        if (matrix.shadeprops !== true) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Shade properties can only be configured for grey klemme L devices"));
            return;
        }

        var icon = 'titleIcon' + dSS.util.genCSSName(record);

        var shadeloaders =
            [
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.SHADE_OPEN_CLOSE,
                    url: '/json/device/getConfigWord',
                    params:
                    {
                        dsid: record.get('dsid'),
                        'class': 3,
                        index: 0x32
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.SHADE_MTIME_BLINK,
                    url: '/json/device/getConfig',
                    params:
                    {
                        dsid: record.get('dsid'),
                        'class': 3,
                        index: 0x11
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.SHADE_CLOSE_OPEN,
                    url: '/json/device/getConfigWord',
                    params:
                    {
                        dsid: record.get('dsid'),
                        'class': 3,
                        index: 0x30
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.SHADE_FTEND_TRESHOLDENDSW,
                    url: '/json/device/getConfigWord',
                    params:
                    {
                        dsid: record.get('dsid'),
                        'class': 3,
                        index: 0x38
                    }
                })
            ];

       if (matrix.motiontimefins === true) {
           shadeloaders.push(
               new dSS.devprops.Loader({
                   loaderId: dSS.devprops.Loaders.SHADE_MTIME_FINS,
                   url: '/json/device/getConfigWord',
                   params:
                   {
                       dsid: record.get('dsid'),
                       'class': 3,
                       index: 0x3e
                   }
               })
           );
       }

       shadeloaders = shadeloaders.concat(
            [
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.SHADE_NUMST_MTIME_FTUNE,
                    url: '/json/device/getConfigWord',
                    params:
                    {
                        dsid: record.get('dsid'),
                        'class': 3,
                        index: 0x3a
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.SHADE_CALIBRATION_ALLOWED,
                    url: '/json/device/getConfig',
                    params:
                    {
                        dsid: record.get('dsid'),
                        'class': 3,
                        index: 0x3d
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH,
                    url: '/json/device/getConfigWord',
                    params:
                    {
                        dsid: record.get('dsid'),
                        'class': 3,
                        index: 0x36
                    }
                })
            ]
        );

        var win = new dSS.devprops.DevicePropertiesWindow({
            name: record.get('name'),
            width: dSS.gthelper.getDialogConfig('shadePropertyWindowWidth'),
            dsid: record.get('dsid'),
            extraTitle: dSS._("shade"),
            showDefaults: true,
            standardForms:
            [
                {
                    xtype: 'shadepropsform',
                    dsid: record.get('dsid'),
                    matrix: matrix
                }
            ],
            advancedForms:
            [
                {
                    xtype: 'shadepropsadvancedform',
                    dsid: record.get('dsid')
                }
            ],
            loaders: shadeloaders,
            iconCls: icon,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },
    showDevicePropertiesWindow: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }
            record = sel.getSelected();
            // it seems that the grid gets out of sync with the store,
            // groups were correctly updated in the store by a previous
            // loadData, but info if the record comes from the selection model,
            // then its out of sync and seems to be an older copy

            record = this.getStore().getById(record.get('dsid'));
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var devInfo = dSS.util.parseProductID(record.get('productID'));
        var type = devInfo[0];
        var number = devInfo[1];
        var cls = dSS.util.parseFunctionID(record.get('functionID'));
        var stdForms = [];
        var advancedForms = [];
        var loaders = [];
        var showDeviceInfo = true;
        var matrix;
        var extraTitle;

        try {
            matrix = dSS.devprops.GetVisibleProperties(cls[0], type, number);
        } catch (error) {
            // for now we can safely assume all as invisible
            matrix = error;
        }

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            var addloader = false;
            var addLocalPrioLoader = false;
            var color = cls[0];

            if (matrix.akmsensor === true) {
                stdForms.push(
                    {
                        xtype: 'dssakmsensorconfigform',
                        dsid: record.get('dsid')
                    });
                loaders.push(new dSS.devprops.Loader({
                             loaderId: dSS.devprops.Loaders.AKM_BINARY_INPUTS,
                             url: '/json/device/getBinaryInputs',
                             params:
                             {
                                 dsid: record.get('dsid')
                             }
                         }));
            }

            if (matrix.akminput === true) {
                stdForms.push(
                    {
                        xtype: 'dssakminputconfigform',
                        dsid: record.get('dsid')
                    });
                loaders.push(new dSS.devprops.Loader({
                             loaderId: dSS.devprops.Loaders.AKM_INPUT_PROPERTY,
                             url: '/json/property/getString',
                             params:
                             {
                                 path: '/apartment/zones/zone0/devices/' +
                                       record.get('dsid') +
                                       '/AKMInputProperty'
                             }
                         }));

            }

            if (matrix.akmdelay === true) {
                stdForms.push(
                    {
                        xtype: 'dssakmdelayconfigform',
                        dsid: record.get('dsid')
                    });
                loaders.push(new dSS.devprops.Loader({
                             loaderId: dSS.devprops.Loaders.AKM_DELAY_CONFIG,
                             url: '/json/device/getAKMInputTimeouts',
                             params:
                             {
                                 dsid: record.get('dsid')
                             }
                         }));
            }

            if ((matrix.jokerconfig === true) &&
                (cls[0] === dSS.config.groups.JOKER)) {
                color = dSS.util.getJokerGroup(record.get('groups'));
                // assume it's not yet configured
                if (color === null) {
                    color = dSS.config.groups.JOKER;
                }
                stdForms.push(
                    {
                        xtype: 'dssdevjokerconfigform',
                        dsid: record.get('dsid'),
                        device: record,
                        product: devInfo[0],
                        group: color
                    });
            }

            if (matrix.optypeconfig === true) {
                stdForms.push(
                    {
                        xtype: 'dssdevoptypeform',
                        dsid: record.get('dsid')
                    });
                addloader = true;
            }

            if ((matrix.pushbutton === true) &&
                (record.get('productRevision') >= dSS.config.PushbuttonMinFW)) {
                stdForms.push(
                    {
                        xtype: 'dssdevgrouppushbuttonform',
                        dsid : record.get('dsid'),
                        product: type,
                        matrix: matrix,
                        group: color
                    });

                if (matrix.pushbadvanced === true) {
                    advancedForms.push(
                        {
                            xtype: 'dssdevgrouppushbuttonadvancedform',
                            dsid : record.get('dsid'),
                            matrix: matrix,
                            group: color
                         });
                } else {
                    stdForms.push(
                    {
                        xtype: 'dssdevgrouppushbuttonadvancedform',
                        dsid : record.get('dsid'),
                        matrix: matrix,
                        group: color,
                        disabled: true,
                        hidden: true
                    });
                }

                addloader = true;
                addLocalPrioLoader = true;
            }

            if ((matrix.pushbutton === true) &&
                (dSS.util.deviceHasMultipleButtons(record.get('functionID')) === true) &&
                ((record.get('buttonInputIndex') === 0) || (record.get('buttonInputIndex') === 2))) {
                stdForms.push(
                {
                    xtype: 'dssdevmultibuttonconfigform',
                    dsid : record.get('dsid'),
                    buttonIndex : record.get('buttonInputIndex'),
                    buttonConfig: record.get('buttonInputMode'),
                    buttonID: record.get('buttonID'),
                    group: color,
                    matrix: matrix
                });
            }

            if (matrix.outmode === true) {
                stdForms.push(
                    {
                        xtype: 'dssdevoutputmodeform',
                        dsid : record.get('dsid')
                    });
                addloader = true;
            }

            if (addloader === true) {
                loaders.push(new dSS.devprops.Loader({
                             loaderId: dSS.devprops.Loaders.COMBINED_PB_OUTMODE,
                             url: '/json/property/query',
                             params:
                             {
                                query: '/apartment/zones/zone0/devices/' +
                                     record.get('dsid') +
                                     '(outputMode)/button(id,setsLocalPriority,activeGroup)'
                             }
                         }));
            }

            if (addLocalPrioLoader === true) {
                loaders.push(new dSS.devprops.Loader({
                             loaderId: dSS.devprops.Loaders.SCENE_CONFIGURATION,
                             url: '/json/device/getSceneMode',
                             params:
                             {
                                 sceneID : dSS.config.scenes.LOCAL_ON,
                                 dsid: record.get('dsid')
                             }
                         }));
            }
        }

        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            var scene;
            try {
                scene = this.findParentByType('dssscenespanel').getSelectedScene();
            } catch (err) {
                Ext.Msg.alert(dSS._("Error"), err);
                return;
            }

            var sceneStore = dSS.data.Loader.getSceneStore();
            var sceneId = record.get('zoneId') + '-' + cls[0] + '-' + scene;
            var sceneRec = sceneStore.getById(sceneId);
            var sceneName = "";
            if (sceneRec !== undefined) {
                sceneName = sceneRec.get('name');
            }

            if (sceneName.length > 0) {
                extraTitle = ' - ' + sceneName;
            } else {
                extraTitle = ' - ' + dSS._(dSS.config.sceneNames[scene]);
            }

            if (dSS.devprops.CheckShowSceneProperties(matrix) === true) {
                // info handled by form
                showDeviceInfo = false;
                stdForms.push({
                            xtype: 'dssdevscenepropsform',
                            dsid : record.get('dsid'),
                            deviceName: record.get('name'),
                            scene: scene,
                            visibilityMatrix: matrix
                       });
            }
        }

        var icon = 'titleIcon' + dSS.util.genCSSName(record);
        var win = new dSS.devprops.DevicePropertiesWindow({
            name: record.get('name'),
            dsid: record.get('dsid'),
            standardForms: stdForms,
            advancedForms: advancedForms,
            loaders: loaders,
            extraTitle: extraTitle,
            iconCls: icon,
            showDefaults: showDeviceInfo,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },

    showCopyWindow: function() {
        var tsvwindow = new dSS.util.GridToSVWindow({
            ignore: [ 'functionID' ],
            gridPanel: this,
            windowWidth: 640,
            windowHeight: 480,
            separator: "\t"
        });
        tsvwindow.show(this);
    },

    editComment: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        Ext.Msg.prompt(
            dSS._("Edit comment"),
            dSS._("Comment for") + ' ' + dSS.util.shortDSID(record.get('dsid')) + ': ',
            function(btn, text) {
            if ((btn === 'ok') && (text !== record.get('comment'))) {
                Ext.Ajax.request({
                    url: '/json/property/setString',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        path:   '/apartment/zones/zone0/devices/' +
                                record.get('dsid') + '/comment',
                        value: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(
                                                        result.responseText);
                            if (jsonData.ok) {
                                Ext.Ajax.request({
                                    url: '/json/property/setFlag',
                                    disableCaching: true,
                                    method: "GET",
                                    params: {
                                        flag: 'ARCHIVE',
                                        path: '/apartment/zones/zone0/devices/'+
                                                record.get('dsid') + '/comment',
                                        value: true
                                    },
                                    success: function(result, request) {
                                        try {
                                            var jsonData = Ext.util.JSON.decode(
                                                result.responseText);
                                            if (jsonData.ok) {
                                                record.set('comment', text);
                                                record.commit();
                                            }
                                        } catch(err) {
                                            Ext.MessageBox.alert(dSS._("Error"),
                                            dSS._("Could not save comment on dSS") +
                                              ": " + err);
                                        }
                                    },
                                    failure: function(result, request) {
                                        Ext.MessageBox.alert(dSS._("Error"),
                                            dSS._("Could not save comment on dSS"));
                                    }
                                });
                            } else {
                                Ext.MessageBox.alert(dSS._("Error"),
                                                    dSS._("Could not add comment"));
                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(dSS._("Error"),
                                     dSS._("Could not add comment") + ": " + err);
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(dSS._("Error"),
                                dSS._("Could not send command to dSS"));
                    }
                });
            }
        },
        this, true, record.get('comment'));
    },
    showConsumptionThresholdWindow: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var devInfo = dSS.util.parseProductID(record.get('productID'));
        var type = devInfo[0];
        var number = devInfo[1];
        var cls = dSS.util.parseFunctionID(record.get('functionID'));
        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(cls[0], type, number);
        } catch (error) {
            // for now we can safely assume all as invisible
            matrix = error;
        }

        if (matrix.consumption !== true) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Consumption thresholds can only be configured for klemme L and ZWS devices"));
            return;
        }

        var icon = 'titleIcon' + dSS.util.genCSSName(record);

        var thresholdloaders =
            [
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_CURRENT,
                    url: '/json/device/getSensorValue',
                    params:
                    {
                        dsid: record.get('dsid'),
                        sensorIndex: 2
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_TABLE_ENTRY,
                    url: '/json/device/getSensorEventTableEntry',
                    params:
                    {
                        dsid: record.get('dsid'),
                        eventIndex: 0
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_TABLE_ENTRY,
                    url: '/json/device/getSensorEventTableEntry',
                    params:
                    {
                        dsid: record.get('dsid'),
                        eventIndex: 1
                    }
                })
            ];

        var win = new dSS.devprops.DevicePropertiesWindow({
            width: 554,
            name: record.get('name'),
            dsid: record.get('dsid'),
            title: dSS._("Configure consumption event"),
            showDefaults: true,
            standardForms:
            [
                {
                    xtype: 'currentconsumptionform',
                    dsid: record.get('dsid')
                },
                {
                    xtype: 'consumptioneventform',
                    dsid: record.get('dsid'),
                    eventIndex: 0,
                    deviceName: record.get('name')
                },
                {
                    xtype: 'consumptioneventform',
                    dsid: record.get('dsid'),
                    eventIndex: 1,
                    deviceName: record.get('name')
                }
            ],
            loaders: thresholdloaders,
            iconCls: icon,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },

    reconfigureColumns: function(mode) {
        var col = this.getColumnModel();
        var i;
        var hide;
        if (mode === 'zone') {
            hide = true;
        } else if (mode === 'apartment') {
            hide = false;
        }

        i = col.findColumnIndex('buttonMode');
        if (i !== -1) {
            col.setHidden(i, hide);
        }

        i = col.findColumnIndex('meterName');
        if (i !== -1) {
            col.setHidden(i, hide);
        }

        i = col.findColumnIndex('zoneName');
        if (i !== -1) {
            col.setHidden(i, hide);
        }

        i = col.findColumnIndex('comment');
        if (i !== -1) {
            col.setHidden(i, hide);
        }
    },

    removeDeviceFromGroup: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }

        if ((this.selectedGroup === undefined) ||
            (this.selectedGroup.get('id') < dSS.config.UserGroupStartIndex)) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("Can not remove device from group, invalid group selected!"));
            return;
        }
        records = sel.getSelections();

        var skippedNotPresent = false;
        var present = [];

        var i;
        for (i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent') === false) {
                skippedNotPresent = true;
                continue;
            }

            present.push(record);
        }

        if (present.length > 0) {
            this.removePresentFromGroup(present, this.selectedGroup.get('id'));
        } else {
            if (skippedNotPresent === true) {
                Ext.Msg.alert(dSS._("Error"), dSS._("Present devices can not be removed!"));
            }
        }
    },
    removePresentFromGroup: function(remainingRecords, groupID) {
        var record = remainingRecords.shift();

        var maskel = this.findParentByType(Ext.Viewport).el;
        maskel.mask(
            dSS.gt.strargs(
                dSS._("Removing device %1 from group %2"),
                [
                    '&quot;' +
                    dSS.util.getName(record.get("name")) +
                    '&quot; (' +
                    dSS.util.shortDSID(record.get("dsid")) +
                    ')',
                    '&quot;' + groupID +  '&quot;'
                ]
            )
        );
        Ext.Ajax.request({
            url: '/json/structure/groupRemoveDevice',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                deviceID: record.get('dsid'),
                groupID: groupID
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        if (jsonData.result.action === "update") {
                            var store = dSS.data.Loader.getDeviceStore();
                            store.loadData(jsonData.result, true);
                        }
                        if (remainingRecords.length > 0) {
                            this.removePresentFromGroup(remainingRecords,
                                                        groupID);
                        } else {
                            maskel.unmask();
                            this.fireEvent('removedfromgroup');
                        }
                    } else {
                        maskel.unmask();
                        this.fireEvent('removedfromgroup');
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not remove device from group"),
                            jsonData.message);
                    }
                }
                catch(err) {
                    maskel.unmask();
                    this.fireEvent('removedfromgroup');
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not send remove device from group command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                maskel.unmask();
                this.fireEvent('removedfromgroup');
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not send remove device from group command to dSS"));
            }
        });
    }
});

Ext.reg('dssdevicepanel', dSS.grid.DevicePanel);

